from nltk_lite.parse import Tree
tagged_tokens = [("the", "DT"), ("little", "JJ"), ("cat", "NN"),
     ("sat", "VBD"), ("on", "IN"), ("the", "DT"), ("mat", "NN")]
Tree('S', tagged_tokens).draw()

from nltk_lite import chunk
cp = chunk.Regexp("NP: {<DT>?<JJ>*<NN>}")

cp.parse(tagged_tokens).draw()

cp.parse(tagged_tokens)

nouns = [("money", "NN"), ("market", "NN"), ("fund", "NN")]
grammar = "NP: {<NN><NN>}  # Chunk two consecutive nouns"
cp = chunk.Regexp(grammar)
print cp.parse(nouns)

print cp1.parse(tagged_tokens, trace=1)

print cp2.parse(tagged_tokens, trace=1)



text = '''
he PRP B-NP
accepted VBD B-VP
the DT B-NP
position NN I-NP
of IN B-PP
vice NN B-NP
chairman NN I-NP
of IN B-PP
Carlyle NNP B-NP
Group NNP I-NP
, , O
a DT B-NP
merchant NN I-NP
banking NN I-NP
concern NN I-NP
. . O
'''
chunk.conllstr2tree(text, chunk_types=('NP',)).draw()

from nltk_lite.corpora import conll2000, extract
print extract(2000, conll2000.chunked())

from nltk_lite.corpora import conll2000, extract
print extract(2000, conll2000.chunked(chunk_types=('NP',)))

cp = chunk.Regexp("")
print chunk.accuracy(cp, conll2000.chunked(chunk_types=('NP',)))

grammar = r"NP: {<[CDJNP].*>+}"
cp = chunk.Regexp(grammar)
print chunk.accuracy(cp, conll2000.chunked(chunk_types=('NP',)))

def chunked_tags(train):
    """Generate a list of tags that tend to appear inside chunks"""
    from nltk_lite.probability import ConditionalFreqDist
    cfdist = ConditionalFreqDist()
    for t in train:
        for word, tag, chtag in chunk.tree2conlltags(t):
            if chtag == "O":
                cfdist[tag].inc(False)
            else:
                cfdist[tag].inc(True)
    return [tag for tag in cfdist.conditions() if cfdist[tag].max() == True]

def baseline_chunker(train):
    import re
    chunk_tags = [re.sub(r'(\W)', r'\\\1', tag)
                  for tag in chunked_tags(train)]
    grammar = 'NP: {<' + '|'.join(chunk_tags) + '>+}'
    return chunk.Regexp(grammar)

cp = baseline_chunker(conll2000.chunked(files='train', chunk_types=('NP',)))
print chunk.accuracy(cp, conll2000.chunked(files='test', chunk_types=('NP',)))

grammar = r"""
NP:
  {<.*>+}          # Chunk everything
  }<VBD|IN>+{      # Chink sequences of VBD and IN
"""
cp = chunk.Regexp(grammar)
print cp.parse(tagged_tokens)

print chunk.accuracy(cp, conll2000.chunked(files='test', chunk_types=('NP',)))

example = extract(2000, conll2000.chunked())
print example

print example.flatten()

example = extract(2000, conll2000.chunked())
print cp.parse(example.flatten(), trace=1)

from nltk_lite import tag
chunk_data = [[(t,c) for w,t,c in chunk.tree2conlltags(chtree)]
             for chtree in conll2000.chunked()]

unigram_chunker = tag.Unigram()
unigram_chunker.train(chunk_data)
print tag.accuracy(unigram_chunker, chunk_data)

bigram_chunker = tag.Bigram(backoff=unigram_chunker)
bigram_chunker.train(chunk_data)
print tag.accuracy(bigram_chunker, chunk_data)

print cp.parse(tagged_tokens)

tagged_tokens = [("John", "NNP"), ("thinks", "VBZ"), ("Mary", "NN"),
    ("saw", "VBD"), ("the", "DT"), ("cat", "NN"), ("sit", "VB"),
    ("on", "IN"), ("the", "DT"), ("mat", "NN")]
print cp.parse(tagged_tokens)

cp = chunk.Regexp(grammar, loop=2)
print cp.parse(tagged_tokens)

